--
-- idata: {
--  "session": 		guid of watchload session	
--	"template":		guid of watchload template
--  "list":			guid of list, unique, optional
--  "parent":		guid of parent list to update, must exist, must be older, optional
--  "name":			| name of list 		|				
--  "owner":		| owner of list		| - unique combination
--  "version":		| version of list	|
--  "checksum": 	hex string of sha256(ordered plates), optional		
--  "memo":			optional
--}
--

create or replace function hwp.watchload ( idata json ) returns json as $$	
import json
import uuid

try:
    data = json.loads(idata)
    if not isinstance(data, dict):
        raise ValueError('Invalid data')
    _sid = data['session']
    _template = data['template']
    _list = data.get('list')
    _parent = data.get('parent')
    _name = data['name']
    _owner = data['owner']
    _version = data['version']
    _checksum = data.get('checksum')
    _memo = data.get('memo')
    _csr = "none"
    total = errors = 0
except Exception as ex:
    plpy.execute("select core.log('hwp.watchload','error','{0}')"
                 .format(str(ex).replace("'", "`")), 0)
    return {"errcode": -1, "errmsg": "Data error: " + str(ex)}

if _list:
    rv = plpy.execute("select * from only hwp.watchlib where listid = {0}"
                      .format(plpy.quote_literal(_list)), 1)
    if rv.nrows() > 0:
        plpy.execute("select core.log('hwp.watchload','error',"
                     "'List with unique id ' || {0} || ' already exists')"
                     .format(plpy.quote_literal(_parent)), 0)
        return {"errcode": 1, "errmsg": "List uuid already loaded"}
else:
    _list = uuid.uuid1()

if _parent:
    rv = plpy.execute("select * from only hwp.watchlib where listid = {0}"
                      .format(plpy.quote_literal(_list)), 1)
    if rv.nrows() < 1:
        plpy.execute("select core.log('hwp.watchload','error',"
                     "'Parent with unique id ' || {0} || ' does not exists')"
                     .format(plpy.quote_literal(_parent)), 0)
        return {"errcode": 1, "errmsg": "Parent uuid does not exists"}
    _name = rv[0]['name']
    _owner = rv[0]['owner']
    if _version <= rv[0]['version']:
        plpy.execute("select core.log('hwp.watchload','error',"
                     "'Version for parent ' || {0} || ' "
                     "is more then updating')"
                     .format(plpy.quote_literal(_parent)), 0)
        return {"errcode": 1, "errmsg": "Version is less then exists"}
else:
    rv = plpy.execute("select * from only hwp.watchlib "
                      "where owner = {0} and name = {1}"
                      .format(plpy.quote_literal(_owner),
                              plpy.quote_literal(_name)), 1)
    if rv.nrows() > 0:
        _parent = rv[0]['listid']
        if _version <= rv[0]['version']:
            plpy.execute("select core.log('hwp.watchload','error',"
                         "'Version for parent ' || {0} || ' "
                         "is more then updating')"
                         .format(plpy.quote_literal(_parent)), 0)
            return {"errcode": 1, "errmsg": "Version is less then exists"}

if _parent:
    plpy.execute("update only hwp.watchlib set active = false "
                 "where listid = {0}".format(plpy.quote_literal(_parent)), 0)

rvt = plpy.execute("select * from hwp.watchload_tpl where id = {0}"
                   "".format(plpy.quote_nullable(_template)), 1)
if rvt.nrows() < 1:
    plpy.execute("select core.log('hwp.watchload','error',"
                 "'Template not found')", 0)
    return {"errcode": 1, "errmsg": "Template not found"}

try:
    _tsep = rvt[0]['separator']
    _tplate = rvt[0]['tp_plate']
    _tcolor = rvt[0]['tp_color']
    _tclass = rvt[0]['tp_class']
    _tcomment = rvt[0]['tp_comment']
    _textra = rvt[0]['tp_extra']
except Exception as ex:
    plpy.execute("select core.log('hwp.watchload','error','{0}')"
                 .format(str(ex).replace("'", "`")), 0)
    return {"errcode": -1, "errmsg": "Template error: " + str(ex)}

try:
    plpy.execute(
        "insert into hwp.watchlib (listid,name,owner,version,checksum,memo)"
        " values({0},{1},{2},{3},{4},{5})".format(
            plpy.quote_nullable(_list),
            plpy.quote_nullable(_name),
            plpy.quote_nullable(_owner),
            plpy.quote_nullable(_version),
            plpy.quote_nullable(_checksum),
            plpy.quote_nullable(_memo)
        ), 0)

    rv = plpy.execute("select * from hwp.watchload_str "
                      "where sid = {0} order by id"
                      .format(plpy.quote_literal(_sid)))
    for r in rv:
        total += 1
        _id = r['id']
        _str = r['str']
        try:
            _arr = (x.strip() for x in _str.split(_tsep))
            rvi = plpy.execute("insert into hwp.watchlist(listid,no,plate,"
                               "color,class,memo,extra) values({},{},{},{},"
                               "{},{},{},{}) returning listid,no"
                               .format(
                    plpy.quote_literal(_list),
                    _id,
                    plpy.quote_literal(_tplate.format(*_arr)),
                    plpy.quote_literal(_tcolor.format(*_arr)),
                    plpy.quote_literal(_tclass.format(*_arr)),
                    plpy.quote_literal(_tmemo.format(*_arr)),
                    plpy.quote_literal(_textra.format(*_arr))
                ), 1)
        except Exception as ex:
            plpy.execute("update hwp.plateload_str set errcode = 1,"
                         " errmsg = {0} where sid = {1} and id = {2}".format(
                    plpy.quote_literal(str(ex)),
                    plpy.quote_literal(_sid), _id), 0)
            errors += 1
    if not _checksum:
        plpy.execute("update hwp.watchlib set "
                     "checksum = hwp.watchlist_sum(listid) where listid = {0}"
                     .format(plpy.quote_literal(_list)), 0)
        _csr = "new"
    else:
        rv = plpy.execute("select hwp.watchlist_sum({0}) = {1}::bytea as ok"
                          .format(plpy.quote_literal(_list),
                                  plpy.quote_literal(_checksum)),
                          1)
        _csr = "failed"
        if rv[0]['ok']:
            _csr = "ok"
except Exception as ex:
    plpy.execute("select core.log('hwp.watchload','error','{0}')"
                 .format(str(ex).replace("'", "`")), 0)
    return {"errcode": -1, "errmsg": "Load error: " + str(ex)}

return {"errcode": 0, "errmsg": "Ok", "total": total,
        "errors": errors, "checksum": _csr}
$$ language plpythonu;
