CREATE OR REPLACE FUNCTION public.plate_recrop(
	i_plateid bigint 
)
  RETURNS text AS
$BODY$

import cStringIO
import json
import os
import sys
import hashlib

from PIL import Image, ImageChops, ImageDraw, ImageFont

def _iinfo(img):
    ii = dict()
    ii['size'] = img.size
    ii['bbox'] = img.getbbox()
    ii['extrema'] = img.getextrema()
    ii['format'] = img.format
    ii['mode'] = img.mode
    ii['extra'] = img.info
    return ii

def _ihash(idata):
    ih = dict()
    ih['size'] = len(idata)
    ih['md5'] = hashlib.md5(idata).hexdigest()
    ih['sha1'] = hashlib.sha1(idata).hexdigest()
    ih['sha256'] = hashlib.sha256(idata).hexdigest()
    return ih

def _titlefont():
    linuxfonts = ["/usr/share/fonts/truetype/ttf-dejavu/DejaVuSans.ttf",
                  "/usr/share/fonts/truetype/dejavu/DejaVuSans.ttf"]
    if 'win' in sys.platform:
        return "arial.ttf"
    if 'linux' in sys.platform:
        return next((f for f in linuxfonts if os.path.isfile(f)), None)
    return None

def _transform(rect, transform, box):
    return [
        (rect[0] * transform['a'] + transform['x']) * box[0],
        (rect[1] * transform['d'] + transform['y']) * box[1],
        (rect[2] * transform['a'] + transform['x']) * box[0],
        (rect[3] * transform['d'] + transform['y']) * box[1]
    ]

def _aint(a):
    return [int(x) if x > 1.0 else int(round(x)) for x in a]

def _subtitle(ims, conf, data, rects, transform, rectmargin):
    try:
        (ims_w, ims_h) = ims.size
    except Exception as ex:
        plpy.execute("select core.log('_subtitle','error','{0}')"
                     .format(str(ex).replace("'", "`")), 0)
        return None
    _DEFAULTCONF = {
        "float": "bottom",
        "height": 0,
        "grid": (3, 3),
        "align": ("left", "center"),
        "padding": (3, 0),
        "font": {"name": "cour.ttf", "size": 14}
    }
    if transform is None:
        transform = {"a": 1, "b": 0, "c": 0, "d": 1, "x": 0, "y": 0}
    try:
        if not isinstance(conf, dict):
            raise ValueError('conf')
        if not isinstance(conf["template"], tuple):
            raise ValueError('template')
        if not isinstance(data, dict):
            raise ValueError('data')
        if not isinstance(rects, tuple):
            raise ValueError('rects')
        if not isinstance(transform, dict):
            raise ValueError('transform')
    except Exception as ex:
        plpy.execute("select core.log('_subtitle arg parser','error','{0}')"
                     .format(str(ex).replace("'", "`")), 0)
        return None
    if 'float' not in conf:
        conf['float'] = _DEFAULTCONF['float']
    if 'height' not in conf:
        conf['height'] = _DEFAULTCONF['height']
    if 'grid' not in conf:
        conf['grid'] = _DEFAULTCONF['grid']
    if 'align' not in conf:
        conf['align'] = _DEFAULTCONF['align']
    if 'padding' not in conf:
        conf['padding'] = _DEFAULTCONF['padding']
    if 'font' not in conf:
        conf['font'] = _DEFAULTCONF['font']
    try:
        font = ImageFont.truetype(conf['font']['name'],
                                  conf['font']['size'], encoding='unic')
    except Exception as ex:
        plpy.execute("select core.log('_subtitle','error','{0}')"
                     .format(str(ex).replace("'", "`")), 0)
        return None
    (fw, fh) = font.getsize('0987654321')
    if conf['height'] == 0:
        conf['height'] = fh * 2 * conf['grid'][0]
    (dx, dy) = (int(ims_w // conf['grid'][1]),
                int(conf['height'] // conf['grid'][0] // 2))
    imt = Image.new("RGB", (ims_w, conf['height']), (0, 0, 0))
    draw = ImageDraw.Draw(imt)

    for tpl in conf["template"]:
        try:
            token = (eval("data" + "".join(map(lambda x: ("['" + x + "']"),
                                               (tpl[4].replace('[', ' ').
                                                replace(']', ' ').split()))))
                     if len(tpl[4]) > 0 else '')
        except:
            token = ''
        txt = tpl[2] + ' ' + token + tpl[3]
        (x, y) = (tpl[1] * dx + conf['padding'][0],
                  ((tpl[0] * 2 + 1) * dy - int(fh // 2)))
        draw.text((x, y), txt.decode('utf-8'), font=font)
    imr = Image.new("RGB", (ims_w, ims_h + conf['height']), (0, 0, 0))
    (imr_w, imr_h) = imr.size
    draw = ImageDraw.Draw(ims)
    for rect in rects:
        rect = _aint(_transform(rect, transform, (ims_w, ims_h)))
        draw.rectangle(_aint([a * (100. + b) / 100. for a, b in
                              zip(list(rect), rectmargin)]), outline='red')
    if conf['float'] == 'bottom':
        imr.paste(ims, (0, 0))
        imr.paste(imt, (0, ims_h))
    else:
        imr.paste(imt, (0, 0))
        imr.paste(ims, (0, conf['height'] + 1))
    tr = {"a": ((ims_w * 1.0) / (imr_w * 1.0)), "b": 0.0, "c": 0.0,
          "d": ((ims_h * 1.0) / (imr_h * 1.0)), "x": 0.0, "y": 0.0}
    return {"data": imr, "transform": tr, "info": _iinfo(imr)}

def _recrop(ims, i_rect, i_margin, i_sizes):
    try:
        (ims_w, ims_h) = ims.size
    except Exception as ex:
        plpy.execute("select core.log('_recrop','error','{0}')".format(ex), 0)
        return None
    if len(i_sizes) == 4:
        (fw, fh, frw, frh, faw, fah) = (i_sizes[0], i_sizes[1], i_sizes[2],
                                        i_sizes[3], None, None)
    elif len(i_sizes) == 6:
        (fw, fh, frw, frh, faw, fah) = (i_sizes[0], i_sizes[1], i_sizes[2],
                                        i_sizes[3], i_sizes[4], i_sizes[5])
    else:
        plpy.execute("select core.log('_recrop','error','invalid sizes')", 0)
        return None

    (fw, fh, frw, frh, faw, fah) = (
        fw if (fw is not None and fw > 0) else None,
        fh if (fh is not None and fh > 0) else None,
        frw if (frw is not None and frw > 0) else None,
        frh if (frh is not None and frh > 0) else None,
        faw,
        fah
    )
    (rl, rt, rr, rb) = (
        int(i_rect[0] * ims_w),
        int(i_rect[1] * ims_h),
        int(i_rect[2] * ims_w),
        int(i_rect[3] * ims_h)
    )
    (rl, rt, rr, rb) = (
        0 if (rl is None or rl < 1 or rl > ims_w or rl >= rr) else rl,
        0 if (rt is None or rt < 1 or rt > ims_h or rt >= rb) else rt,
        0 if (rr is None or rr < 1 or rr > ims_w or rr <= rl) else rr,
        0 if (rb is None or rb < 1 or rb > ims_h or rb <= rt) else rb
    )
    if len(i_margin) == 4:
        (lmcw, lmch, tmcw, tmch, rmcw, rmch, bmcw, bmch) = (i_margin[0], 0.0,
                                                            i_margin[1], 0.0,
                                                            i_margin[2], 0.0,
                                                            i_margin[3], 0.0)
    elif len(i_margin) == 8:
        (lmcw, lmch, tmcw, tmch, rmcw, rmch, bmcw, bmch) = (
            i_margin[0],
            i_margin[1],
            i_margin[2],
            i_margin[3],
            i_margin[4],
            i_margin[5],
            i_margin[6],
            i_margin[7]
        )
    else:
        plpy.execute("select core.log('_recrop','error',"
                     "'invalid margin sizes')", 0)
        return None
    (lmcw, lmch, tmcw, tmch, rmcw, rmch, bmcw, bmch) = (
        0 if (lmcw is None) else lmcw,
        0 if (lmch is None) else lmch,
        0 if (tmcw is None) else tmcw,
        0 if (tmch is None) else tmch,
        0 if (rmcw is None) else rmcw,
        0 if (rmch is None) else rmch,
        0 if (bmcw is None) else bmcw,
        0 if (bmch is None) else bmch
    )
    if ((rr - rl) < 1 or (rr - rl) > ims_w - 1 or (rb - rt) < 1 or
            (rb - rt) > ims_h - 1):
        plpy.execute("select core.log('_recrop','error',"
                     "'invalid rect borders')", 0)
        return None
    sw = (rr - rl)
    sh = (rb - rt)
    lm = int(sw * lmcw + sh * lmch)
    rm = int(sw * rmcw + sh * rmch)
    tm = int(sw * tmcw + sh * tmch)
    bm = int(sw * bmcw + sh * bmch)
    bw = sw + lm + rm
    bh = sh + tm + bm
    kb = bw * 1.0 / bh
    if bw < 1 or bh < 1:
        plpy.execute("select core.log('_recrop','error',"
                     "'invalid body borders')", 0)
        return None
    adapt = False
    if faw or fah:
        adapt = True
        if faw == 0:
            faw = bw
        if fah == 0:
            fah = bh
        fw = faw
        fh = fah
    if (fw and (fw < 2 or fw > ims_w - 2)) or (fh and
                                               (fh < 2 or fh > ims_h - 2)):
        plpy.execute("select core.log('_recrop','error',"
                     "'invalid body sizes')", 0)
        return None
    kr = None
    if frw:
        kr = frw * 1.0 / sw
    if frh:
        kr = frh * 1.0 / sh
    if frw and frh:
        kr = max(frw * 1.0 / sw, frh * 1.0 / sh)
    if fw and fh:
        kf = fw * 1.0 / fh
        (mw, mh) = ((lm + rm), (tm + bm))
        if not kb == kf:
            if bw >= bh:
                lm = (int(round(lm * (1 - (bw - bh * kf) * 1.0 / mw)))
                      if lm > 0 else 0)
                rm = (int(round(rm * (1 - (bw - bh * kf) * 1.0 / mw)))
                      if rm > 0 else 0)
            else:
                tm = (int(round(tm * (1 - (bh - bw * 1.0 / kf) * 1.0 / mh)))
                      if tm > 0 else 0)
                bm = (int(round(bm * (1 - (bh - bw * 1.0 / kf) * 1.0 / mh)))
                      if bm > 0 else 0)
    bw = sw + lm + rm
    bh = sh + tm + bm
    kb = bw * 1.0 / bh
    (mw, mh) = ((lm + rm), (tm + bm))
    (kaw, kah) = ((ims_w - 1) * 1.0 / bw, (ims_h - 1) * 1.0 / bh)
    if min(kaw, kah) < 1:
        lm = int(round(lm * (1 - (bw * (1.0 - kaw)) / mw))) if lm > 0 else 0
        rm = int(round(rm * (1 - (bw * (1.0 - kaw)) / mw))) if rm > 0 else 0
        tm = int(round(tm * (1 - (bh * (1.0 - kah)) / mh))) if tm > 0 else 0
        bm = int(round(bm * (1 - (bh * (1.0 - kah)) / mh))) if bm > 0 else 0
    bw = sw + lm + rm
    bh = sh + tm + bm
    kb = bw * 1.0 / bh
    (cl, ct, cr, cb) = (
        rl - lm,
        rt - tm,
        rr + rm,
        rb + bm
    )
    xo = yo = 0
    if adapt:
        i = 0
        while any([cl < 0, ct < 0, cr > ims_w - 1, cb > ims_h - 1]) and i < 16:
            i += 1
            if cl < 0:
                cr = cr - cl - 1
                cl = 0
            if ct < 0:
                cb = cb - ct - 1
                ct = 0
            if cr > ims_w - 1:
                cl = cl - cr + ims_w
                cr = ims_w - 1
            if cb > ims_h - 1:
                ct = ct - cb + ims_h
                cb = ims_h - 1
        if i > 14:
            plpy.execute("select core.log('_recrop','warning',"
                         "'margin adaptor iterations exceed')", 0)
    else:
        if cl < 0:
            xo = -cl
            cl = 0
        if ct < 0:
            yo = -ct
            ct = 0
        if cr > ims_w - 1:
            cr = ims_w - 1
        if cb > ims_h - 1:
            cb = ims_h - 1

    imp = ims.crop((cl, ct, cr, cb))
    (imp_w, imp_h) = imp.size
    if xo == 0 and yo == 0 and imp_w == bw and imp_h == bh:
        imd = imp
        (imd_w, imd_h) = (imp_w, imp_h)
    else:
        imd = Image.new("RGB", (bw, bh), (0, 0, 0))
        imd.paste(imp, (xo, yo))
        (imd_w, imd_h) = imd.size

    (ww, wh) = (imd_w, imd_h)
    if fw and not fh:
        wh = int(round(wh * fw * 1.0 / ww))
        ww = fw
    if fh and not fw:
        ww = int(round(ww * fh * 1.0 / wh))
        wh = fh
    if fw and fh:
        (ww, wh) = (fw, fh)
    if kr:
        kw = max(ww * 1.0 / imd_w, wh * 1.0 / imd_h)
        if kr > kw:
            ww = int(round(ww * (1 + kr - kw)))
            wh = int(round(wh * (1 + kr - kw)))
    if ww != imd_w or wh != imd_h:
        if wh > 100:
            imd = imd.resize((ww, wh), Image.NEAREST)
        else:
            imd = imd.resize((ww, wh), Image.BICUBIC)
    fr = [cl / (ims_w * 1.0), ct / (ims_h * 1.0), cr / (ims_w * 1.0),
          cb / (ims_h * 1.0)]
    tr = {"a": ((ims_w * 1.0) / (imd_w * 1.0)), "b": 0.0,
          "c": 0.0, "d": ((ims_h * 1.0) / (imd_h * 1.0)),
          "x": (xo - cl) / (imd_w * 1.0), "y": (yo - ct) / (imd_h * 1.0)}
    return {"data": imd, "transform": tr, "outerframe": fr,
            "info": _iinfo(imd)}

"""
	########################################### main()
"""

rv = plpy.execute("""select
        p.post_id, p.channel,
        p.media_id as mid,
        array[p.rect_left,p.rect_top,p.rect_right,p.rect_bottom]::float8[] as rect,
        public.plate_titledata(p.id) as titledata,
        _str('subtitle',_o('platear_stampedframe_template')||'.frame') as titleframecfg,
        _str('subtitle',_o('platear_stampedbody_template')||'.body') as titlebodycfg
        from platear p where p.id = """ + str(i_plateid), 1)
if rv.nrows() < 1:
    return "NO PLATE FOUND [" + str(i_plateid) + "]"

try:
    post = rv[0]['post_id']
    channel = rv[0]['channel']
    mid = rv[0]['mid']
    rect = rv[0]['rect']
    titledata = eval(rv[0]['titledata'])
    titleframecfg = eval(rv[0]['titleframecfg'])
    titlebodycfg = eval(rv[0]['titlebodycfg'])
except Exception as ex:
    plpy.execute("select core.log('recropall plateload','error','{0}')"
                 .format(str(ex).replace("'", "`")), 0)
    return "plate load error"

if mid is None:
    return "NO MEDIA ID FOUND [{0}]".format(mid)

rv = plpy.execute("select plate_image_get('{0}',{1},{2},'frame') as mdata;"
                  .format(post, channel, mid), 1)
if rv.nrows() < 1 or rv[0]['mdata'] is None:
    return "NO MEDIA FOUND [{0}]".format(mid)

try:
    img = Image.open(cStringIO.StringIO(rv[0]['mdata'])).convert("RGB")
except Exception as ex:
    plpy.execute("select core.log('recropall imopen','error','{0}')"
                 .format(str(ex).replace("'", "`")), 0)
    return "image load error"

# (imgW,imgH) = img.size
# if any([x>1 for x in rect]):
#   rect = [r/(s*1.0) for (r,s) in zip(rect,[imgW,imgH,imgW,imgH])]
#   plpy.execute(("update platear set rect_left = %f, rect_top = %f, "
#                 "rect_right = %f, rect_bottom = %f where id = " +
#                 str(i_plateid)) % tuple(rect),0)

rv = plpy.execute("""select
        json(format('{"body":%s,"plate":%s,"stampedframe":%s,"stampedbody":%s}',
        to_json(coalesce(_o('plateimage_store_image_body'),'')),
        to_json(coalesce(_o('plateimage_store_image_plate'),'')),
        to_json(coalesce(_o('plateimage_store_image_stampedframe'),'')),
        to_json(coalesce(_o('plateimage_store_image_stampedbody'),''))
        )) as stores,
        array [
        _o('platear_crop_body_margin_left_w')::float8,
        _o('platear_crop_body_margin_left_h')::float8,
        _o('platear_crop_body_margin_top_w')::float8,
        _o('platear_crop_body_margin_top_h')::float8,
        _o('platear_crop_body_margin_right_w')::float8,
        _o('platear_crop_body_margin_right_h')::float8,
        _o('platear_crop_body_margin_bottom_w')::float8,
        _o('platear_crop_body_margin_bottom_h')::float8 ]::float8[] as bodymargin,
        array [ 0, 0, 0, 0,
            _o('platear_crop_body_width')::integer,
            _o('platear_crop_body_height')::integer
        ]::integer[] as bodysizes,
        array [ 0, 0, 0, 0,
            _o('platear_stamped_body_width')::integer,
            _o('platear_stamped_body_height')::integer
        ]::integer[] as stampedbodysizes,
        array[0.0, 0.0, 0.0, 0.0]::float8[] as platemargin,
        array[_o('platear_crop_plate_width')::integer, _o('platear_crop_plate_height')::integer, 0, 0, 0, 0]::integer[] as platesizes,
        _o('platear_stampedbody_rect_margin')::float8[] as platear_stampedbody_rect_margin,
        _o('platear_stampedframe_rect_margin')::float8[] as platear_stampedframe_rect_margin
    ;""", 1)

if rv.nrows() < 1:
    return "NO OPT FOUND []"

try:
    stores = eval(rv[0]['stores'])
    bodymargin = rv[0]['bodymargin']
    bodysizes = rv[0]['bodysizes']
    stampedbodysizes = rv[0]['stampedbodysizes']
    platemargin = rv[0]['platemargin']
    platesizes = rv[0]['platesizes']
    platear_stampedbody_rect_margin = rv[0]['platear_stampedbody_rect_margin']
    platear_stampedframe_rect_margin = rv[0]['platear_stampedframe_rect_margin']
except Exception as ex:
    plpy.execute("select core.log('recropall optload','error','{0}')"
                 .format(str(ex).replace("'", "`")), 0)
    return "opt load error"

io = cStringIO.StringIO()
imsetq = plpy.prepare("select plate_image_set($1::uuid,$2,$3,$4,true,$5)",
                      ["text", "int4", "int8", "text", "bytea"])
mtsetq = plpy.prepare("select plate_image_reset_meta($1::uuid,"
                      "$2,$3,$4,$5,json($6))",
                      ["text", "int4", "int8", "text", "text", "text"])

if not stores['plate'] and (stores['stampedframe'] or stores['stampedbody']):
    stores['plate'] = '@nosave'
if not stores['body'] and stores['stampedframe']:
    stores['body'] = '@nosave'

imgplate = None
if stores['plate']:
    imgplate = _recrop(img, rect, platemargin, platesizes)
    if not imgplate:
        return "plate recrop error"
    if not stores['plate'] == '@nosave':
        try:
            io.seek(0)
            io.truncate(0)
            imgplate['data'].save(io, "JPEG")
            plpy.execute(imsetq, [post, channel, mid, stores['plate'],
                                  io.getvalue()], 0)
            plpy.execute(mtsetq,
                         [post, channel, mid, stores['plate'], "/info",
                          json.dumps(imgplate['info'])], 0)
            plpy.execute(mtsetq, [post, channel, mid, stores['plate'],
                                  "/transform",
                                  json.dumps(imgplate['transform'])], 0)
            plpy.execute(mtsetq,
                         [post, channel, mid, stores['plate'], "/outer",
                          json.dumps(imgplate['outerframe'])], 0)
            plpy.execute(mtsetq,
                         [post, channel, mid, stores['plate'], "/digest",
                          json.dumps(_ihash(io.getvalue()))], 0)
        except Exception as ex:
            plpy.execute("select core.log('recropall imsave plate',"
                         "'error','{0}')".format(str(ex).replace("'", "`")), 0)
            return "image plate save error"

imgbody = None
if stores['body']:
    imgbody = _recrop(img, rect, bodymargin, bodysizes)
    if not imgbody:
        return "body recrop error"
    if not stores['body'] == '@nosave':
        try:
            io.seek(0)
            io.truncate(0)
            imgbody['data'].save(io, "JPEG")
            plpy.execute(imsetq, [post, channel, mid, stores['body'],
                                  io.getvalue()], 0)
            plpy.execute(mtsetq,
                         [post, channel, mid, stores['body'], "/info",
                          json.dumps(imgbody['info'])], 0)
            plpy.execute(mtsetq,
                         [post, channel, mid, stores['body'], "/transform",
                          json.dumps(imgbody['transform'])], 0)
            plpy.execute(mtsetq,
                         [post, channel, mid, stores['body'], "/outer",
                          json.dumps(imgbody['outerframe'])], 0)
            plpy.execute(mtsetq,
                         [post, channel, mid, stores['body'], "/digest",
                          json.dumps(_ihash(io.getvalue()))], 0)
            if imgplate:
                plpy.execute(mtsetq,
                             [post, channel, mid, stores['body'],
                              "/frames/plate",
                              json.dumps(_transform(imgplate['outerframe'],
                                                    imgbody['transform'],
                                                    (1, 1)))], 0)
        except Exception as ex:
            plpy.execute("select core.log('recropall imsave body','error',"
                         "'{0}')".format(str(ex).replace("'", "`")), 0)
            return "image body save error"

if img:
    plpy.execute(mtsetq, [post, channel, mid, 'frame', "/info",
                          json.dumps(_iinfo(img))], 0)
if imgplate:
    plpy.execute(mtsetq, [post, channel, mid, 'frame', "/frames/plate",
                          json.dumps(imgplate['outerframe'])], 0)
if imgbody:
    plpy.execute(mtsetq, [post, channel, mid, 'frame', "/frames/body",
                          json.dumps(imgbody['outerframe'])], 0)

titlefont = _titlefont()

if "name" not in titleframecfg["font"]:
    titleframecfg["font"]["name"] = titlefont
if "name" not in titlebodycfg["font"]:
    titlebodycfg["font"]["name"] = titlefont

if stores['stampedframe']:
    imgtitled = _subtitle(
        img.copy(),
        titleframecfg,
        titledata,
        (
            imgplate['outerframe'],
            imgbody['outerframe'],
        ),
        None,
        platear_stampedframe_rect_margin
    )
    if not imgtitled:
        return "frame title error"
    try:
        io.seek(0)
        io.truncate(0)
        imgtitled['data'].save(io, "JPEG")
        plpy.execute(imsetq, [post, channel, mid, stores['stampedframe'],
                              io.getvalue()], 0)
        plpy.execute(mtsetq,
                     [post, channel, mid, stores['stampedframe'], "/info",
                      json.dumps(imgtitled['info'])], 0)
        plpy.execute(mtsetq, [post, channel, mid, stores['stampedframe'],
                              "/transform",
                              json.dumps(imgtitled['transform'])], 0)
        plpy.execute(mtsetq,
                     [post, channel, mid, stores['stampedframe'], "/digest",
                      json.dumps(_ihash(io.getvalue()))], 0)
    except Exception as ex:
        plpy.execute("select core.log('recropall stt save','error','{0}')"
                     .format(str(ex).replace("'", "`")), 0)
        return "save title frame error"

if stores['stampedbody']:
    imgcrop = _recrop(
        img,
        rect,
        bodymargin,
        stampedbodysizes
    )
    if not imgcrop:
        return "crop body to title"
    imgtitled = _subtitle(
        imgcrop['data'],
        titlebodycfg,
        titledata,
        (imgplate['outerframe'],),
        imgcrop['transform'],
        platear_stampedbody_rect_margin
    )
    if not imgtitled:
        return "body titile error"
    try:
        io.seek(0)
        io.truncate(0)
        imgtitled['data'].save(io, "JPEG")
        plpy.execute(imsetq, [post, channel, mid, stores['stampedbody'],
                              io.getvalue()], 0)
        plpy.execute(mtsetq,
                     [post, channel, mid, stores['stampedbody'], "/info",
                      json.dumps(imgtitled['info'])], 0)
        plpy.execute(mtsetq, [post, channel, mid, stores['stampedbody'],
                              "/transform",
                              json.dumps(imgtitled['transform'])], 0)
        plpy.execute(mtsetq,
                     [post, channel, mid, stores['stampedbody'], "/digest",
                      json.dumps(_ihash(io.getvalue()))], 0)
    except Exception as ex:
        plpy.execute("select core.log('recropall stb','error','')"
                     .format(str(ex).replace("'", "`")), 0)
        return "save title body error"

return "OK"

$BODY$
  LANGUAGE plpythonu;

