CREATE TABLE if not exists platear
(
	id bigserial primary key,
	guid uuid,
	wtime timestamptz DEFAULT now(),
	ctime timestamptz DEFAULT now(),
	stime timestamptz DEFAULT now(),
	etime timestamptz DEFAULT now(),
	scrollid bigint,
	scrollts timestamp with time zone,
	media_id bigint,
	post_id uuid NOT NULL,
	channel integer NOT NULL DEFAULT 0,
	rect_left float8,
	rect_top float8,
	rect_right float8,
	rect_bottom float8,
	"number" text,
	country integer NOT NULL DEFAULT -1,
	numtype integer NOT NULL DEFAULT -1,
	county integer NOT NULL DEFAULT -1,
	eventspec integer,
	pid bigint,
	ppid bigint,
	iodir integer,
	direction integer not null default 0,
	geodata geodata,
	velocity integer, 
	validity integer NOT NULL DEFAULT 100,
	alarm_type integer NOT NULL DEFAULT 0,
	alarm_state integer NOT NULL DEFAULT 0,
	processing integer not null default 0,
	status integer not null default 0,
	int0 integer,
	int1 integer,
	int2 integer,
	int3 integer,
	comment text NOT NULL DEFAULT ''::text,
	vehicle_length integer NOT NULL DEFAULT -1,
	lane integer NOT NULL DEFAULT 0 
);

DO $$
BEGIN
IF to_regclass('public.i_platear_alarm_state') IS NULL THEN
    CREATE INDEX i_platear_alarm_state ON platear USING btree (alarm_state);
END IF;
IF to_regclass('public.i_platear_alarm_type') IS NULL THEN
    CREATE INDEX i_platear_alarm_type ON platear USING btree (alarm_type);
END IF;
IF to_regclass('public.i_platear_alarm_0') IS NULL THEN
    CREATE INDEX i_platear_alarm_0 ON platear USING btree (alarm_type) WHERE alarm_type = 0;
END IF;
IF to_regclass('public.i_platear_alarm_1') IS NULL THEN
    CREATE INDEX i_platear_alarm_1 ON platear USING btree (alarm_type) WHERE (alarm_type & 1) = 1;
END IF;
IF to_regclass('public.i_platear_alarm_2') IS NULL THEN
    CREATE INDEX i_platear_alarm_2 ON platear USING btree (alarm_type) WHERE (alarm_type & 2) = 2;
END IF;
IF to_regclass('public.i_platear_channel') IS NULL THEN
    CREATE INDEX i_platear_channel ON platear USING btree (channel);
END IF;
IF to_regclass('public.i_platear_guid') IS NULL THEN
    CREATE INDEX i_platear_guid ON platear USING btree (guid);
END IF;
IF to_regclass('public.i_platear_int_0') IS NULL THEN
    CREATE INDEX i_platear_int_0 ON platear USING btree (int0);
END IF;
IF to_regclass('public.i_platear_int_1') IS NULL THEN
    CREATE INDEX i_platear_int_1 ON platear USING btree (int1);
END IF;
IF to_regclass('public.i_platear_int_2') IS NULL THEN
    CREATE INDEX i_platear_int_2 ON platear USING btree (int2);
END IF;
IF to_regclass('public.i_platear_int_3') IS NULL THEN
    CREATE INDEX i_platear_int_3 ON platear USING btree (int3);
END IF;
IF to_regclass('public.i_platear_post') IS NULL THEN
    CREATE INDEX i_platear_post ON platear USING btree (post_id);
END IF;
IF to_regclass('public.i_platear_velocity') IS NULL THEN
    CREATE INDEX i_platear_velocity ON platear USING btree (velocity);
END IF;
IF to_regclass('public.platear_ctime_idx') IS NULL THEN
    CREATE INDEX platear_ctime_idx ON platear USING btree (ctime DESC);
END IF;
--IF to_regclass('public.platear_stime_idx') IS NULL THEN
--    CREATE INDEX platear_stime_idx ON platear USING btree (stime);
--END IF;
--IF to_regclass('public.platear_etime_idx') IS NULL THEN
--    CREATE INDEX platear_etime_idx ON platear USING btree (etime);
--END IF;
IF to_regclass('public.platear_number_btree_idx') IS NULL THEN
    CREATE INDEX platear_number_btree_idx ON platear USING btree (number COLLATE pg_catalog."default");
END IF;
IF to_regclass('public.platear_number_trgm_idx') IS NULL THEN
    CREATE INDEX platear_number_trgm_idx ON platear USING gin (number COLLATE pg_catalog."default" ct.gin_trgm_ops);
END IF;
IF to_regclass('public.platear_pid_btree_idx') IS NULL THEN
    CREATE INDEX platear_pid_btree_idx ON platear USING btree (pid);
END IF;
IF to_regclass('public.platear_scroll_idx') IS NULL THEN
    CREATE INDEX platear_scroll_idx ON platear USING btree (scrollts, scrollid);
END IF;
END$$;

